#!/usr/bin/perl -w
#####################################################################################################################################
#
#PRE Mapper is a bioinformatic tool written in Perl for searching Polycomb Response Elements (PREs) in Drosophila.
#
#Copyright (C) 2019  Arumugam Srinivasan & Rakesh K Mishra
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
######################################################################################################################################
#Usage: perl PRE_Mapper-basic-v1.0.pl <input_fasta_file>
#
#last modified on 25th Feb 2019.
######################################################################################################################################





use strict;
use warnings;



        my $usage1 = "provide a input fasta file \n\n usage: perl PRE_Mapper-basic-v1.0.pl <input_fasta_file> \n";
       
	   
	   
	   #to get the input files
        my $input_fasta_file_name = shift or die $usage1;
        
		
		
		my $detailed_output_file_name = 'PRE_motifs_and_summary.txt';
        my $sequence_out_file_name= 'PRE_sequences.txt';
        my $table_output_file_name= 'PRE_table.txt';
        my $table_to_graphics_output_file_name= 'PRE_table_graphics_out.txt';
        


        my $summary_file_with_path= 'Summary_file.txt';
        #########################to print the summary_table_file#################################################################################
        open(OUTFILE, ">" . $summary_file_with_path) or die ("Cannot open file for output: $!");			      
        close(OUTFILE) or die ("Cannot close file for output: $!");
        

		
		my$hits_table_file_name = 'hits_table.txt';
        my$hits_table_file_name_with_path = "$hits_table_file_name";
        #print"\n$hits_table_file_name_with_path\n";
        
        
        
        
        my$overlapping_unique_hits_file_name ='hits_tables_sorted_and_duplicate_removed.txt';
        my$overlapping_unique_hits_file_name_with_path = "$overlapping_unique_hits_file_name";
        #print"\n$overlapping_unique_hits_file_name_with_path\n";
        
        
        
        
        #pre-set parameters common to all PRE types
        my $window_size = 750;
        my $window_increment = 10;
        
        
        
        
        
        
        

  
        
        
        
        
      
      
        #to initiate undefined varaibles (latter used for comparision between first and second cluster)
        my$first_cluster_start=0;
        my$first_cluster_stop=0;
        my$first_cluster_length=0;
        
        #a flag setting to identify, if the hit is found or not(value of 1 means finding of a hit)
        my$hit_found=0;
        
        
        #initiation for return values from PRE predict subroutine
        my$hit_found_returned=0;
        my$cluster_start_returned=0;
        my$cluster_stop_returned=0;
        my$PRE_id_returned=0;
        my$cluster_length_returned=0;
        
        
        
        
        
        my$current_seq_length_in_full_750bp_window=0;
        
        
       
 
        
        
	
	######################################## PRE Prediction part starts ########################################################
			
        #Now open the file specified by $input_fasta_file_name
	open (INFILE, $input_fasta_file_name) or die( "Cannot open file for input: $!" );
	$/ = undef;
	my $sequenceEntry = <INFILE>;              
	
        close(INFILE)or die ("Cannot close file for output: $!");
        
        my $sequenceTitle;
        
            if ($sequenceEntry =~ m/^>([^\n\cM]+)/)
				
                {
                    $sequenceTitle = $1;
                    $sequenceEntry =~ s/^>[^\n\cM]+//;
                }
			
	    else
                    
                {
    		    $sequenceTitle = "No_title_was_found";
                    
                }
                        
			
	$sequenceEntry =~ s/[^gatcryswkmbdhvn]//ig;
	my $sequenceLength = length($sequenceEntry);
                        
			
	#Write header information to the file specified by $table_output_file_name.
	open(OUTFILE, ">" . $table_output_file_name) or die ("Cannot open file for output: $!");
	print(OUTFILE "PRE\tStart-Stop\t\tSize\t\tTotal Motifs\tDistance (kb)\n");
	close(OUTFILE) or die ("Cannot close file for output: $!");
                        
                        
        #Write header information to the file specified.
	open(OUTFILE, ">" . $detailed_output_file_name) or die ("Cannot open file for output: $!");
	print(OUTFILE "**********************************************************************************\n");
	print(OUTFILE "******************************* Result of PRE Mapper Tool *******************************\n");  
	print(OUTFILE "***********************AUTHORS: A.SRINIVASAN & RAKESH K MISHRA********************\n\n");
	print(OUTFILE "INPUT FILE USED: $input_fasta_file_name \n");
	print(OUTFILE "SEQUENCE TITLE: $sequenceTitle, SEQUENCE LENGTH: $sequenceLength.\n\n");
	close(OUTFILE) or die ("Cannot close file for output: $!");
                        
                        
	#to create sequence ouput file		
	open(OUTFILE, ">" . $sequence_out_file_name) or die ("Cannot open file for output: $!");
	close(OUTFILE) or die ("Cannot close file for output: $!");
			
	#to create all_hits_table.txt file which will be used by sort_hits.pl	
	open(OUTFILE, ">" . $hits_table_file_name_with_path) or die ("Cannot open file for output: $!");
	print(OUTFILE "##Name\t\tID-No\tStart\tStop\n");
	close(OUTFILE) or die ("Cannot close file for output: $!");
 	
        
        ###Start of prediction process
	my$PRE_id_start=0;
	#default start of sequence
        my$start=0;    
        #default value for end of sequence
        my$end=0;
       
       
        #PRE search or prediction is performed while the start sequence is less than or equal to sequence length               
        while ($start<=$sequenceLength)
        {
            
			#end of a given window calculated by adding the window size
			$end=$start+$window_size;
                        
                        
                        
			#to limit the end sequence to the maximum seq length
			if($end>=$sequenceLength)
			{$end=$sequenceLength;}
                        
                        my$current_window_sequence='';
                        
			#to get current seq
			$current_window_sequence = substr($sequenceEntry, $start, $end - $start);
			
                        
                        my$current_seq_length=0;
                        $current_seq_length = length($current_window_sequence);
			$current_seq_length_in_full_750bp_window=$current_seq_length;
                        
                        
                        
                        my$transition_sequence='';
                        
                        
                        
                        #to assigin the $current_window_sequence to $transition_sequence
                        $transition_sequence=$current_window_sequence;
                       
                        
                        
                        #complement the sequence
                        $transition_sequence=~ tr/gatcryswkmbdhvnGATCRYSWKMBDHVN/ctagyrswmkvhdbnCTAGYRSWMKVHDBN/;
                        
                        #reverse the sequence to get the reverse complement
                        
                        my$current_window_reverse_complement_sequence='';
                        $current_window_reverse_complement_sequence=reverse($transition_sequence);
                        
                        
                        
                        
                        #750 window call of PRE_predict sub
                        ######################################################
                        ######################################################
                        ($hit_found_returned,$cluster_start_returned,$cluster_stop_returned,$PRE_id_returned,$cluster_length_returned)=PRE_predict (
                                                                                                                                                             $sequenceTitle,
                                                                                                                                                              $start,
                                                                                                                                                              $end,
                                                                                                                                                              $current_window_sequence,
                                                                                                                                                              $current_window_reverse_complement_sequence,
                                                                                                                                                              $current_seq_length,
                                                                                                                                                              $first_cluster_start,
                                                                                                                                                              $first_cluster_stop,
                                                                                                                                                              $PRE_id_start,
                                                                                                                                                              $first_cluster_length,
                                                                                                                                                              
                                                                                                                                                              
                                                                                                                                                              );
                        
                        #to check if the hit is found or not
                        $hit_found=$hit_found_returned;
                        
                        if($hit_found==1){
                                             
                                            $first_cluster_start=$cluster_start_returned;
                                            $first_cluster_stop=$cluster_stop_returned;			
                                            $PRE_id_start=$PRE_id_returned;
                                            $first_cluster_length=$cluster_length_returned;
                                          
                                          }
                                
                                
                                
                            #start of 500A sub window
                            if ($hit_found==0)
                        
                            {
                                
                                #to get the end postion of first 500bp out of 750 bp window, here end is start+500, start remains the same
                                my$end_500A=$start+500;
                                
                                #to limit the end sequence to the maximum seq length
                                if($end_500A>=$sequenceLength)
                                {$end_500A=$sequenceLength;}
                                
                                
                                #to get current seq
                                $current_window_sequence = substr($sequenceEntry, $start, $end_500A - $start);
                                $current_seq_length = length($current_window_sequence);
                                
                                
                                #to get the reverse complement of current seq
                                #to assigin the $current_window_sequence to $transition_sequence
                                $transition_sequence=$current_window_sequence;
                                
                                
                                
                                #complement the sequence
                                $transition_sequence=~ tr/gatcryswkmbdhvnGATCRYSWKMBDHVN/ctagyrswmkvhdbnCTAGYRSWMKVHDBN/;
                                
                                
                                
                                #reverse the sequence to get the reverse complement
                                $current_window_reverse_complement_sequence=reverse($transition_sequence);
                                
                                
                                #500A window call of PRE_predict sub
                                ######################################################
                                ######################################################
                                ($hit_found_returned,$cluster_start_returned,$cluster_stop_returned,$PRE_id_returned,$cluster_length_returned)=PRE_predict (
                                                                                                                                                                        
                                                                                                                                                                        $sequenceTitle,
                                                                                                                                                                        $start,
                                                                                                                                                                        $end_500A,
                                                                                                                                                                        $current_window_sequence,
                                                                                                                                                                        $current_window_reverse_complement_sequence,
                                                                                                                                                                        $current_seq_length,
                                                                                                                                                                        $first_cluster_start,
                                                                                                                                                                        $first_cluster_stop,
                                                                                                                                                                        $PRE_id_start,
                                                                                                                                                                        $first_cluster_length 
                                                                                                                                                                      );
                                
                                
                                $hit_found=$hit_found_returned;
                                
                                        if($hit_found==1)
                                        
                                            {
                                            
                                            $first_cluster_start=$cluster_start_returned;
                                            $first_cluster_stop=$cluster_stop_returned;			
                                            $PRE_id_start=$PRE_id_returned;
                                            $first_cluster_length=$cluster_length_returned;
                                           
                                            }
                                
                            }#end of 500A sub window
           
           
           
           
           
           
            
                                    #start of 500B sub window
                                    #this part is evaluvated if 500A fails to detect any boudary, start is changed as $start+250, end is same as in 750 bp
                                    if (($hit_found==0) && ($current_seq_length_in_full_750bp_window>=500))
                                        
                                    {
                                        #window size is 5000bp so the end is start + 1000
                                        my$start_500B=$start+250;
                                        #to limit the end sequence to the maximum seq length
                                        
                                        
                                        if($end>=$sequenceLength)
                                        {$end=$sequenceLength;}
                                        
                                        
                                        #to get current seq
                                        $current_window_sequence = substr($sequenceEntry, $start_500B, $end - $start_500B);
                                        $current_seq_length = length($current_window_sequence);
                                        
                                        
                                        #to get the reverse complement of current seq
                                        #to assigin the $current_window_sequence to $transition_sequence
                                        $transition_sequence=$current_window_sequence;
                                        
                                        
                                        #complement the sequence
                                        $transition_sequence=~ tr/gatcryswkmbdhvnGATCRYSWKMBDHVN/ctagyrswmkvhdbnCTAGYRSWMKVHDBN/;
                                        
                                        
                                        #reverse the sequence to get the reverse complement
                                        $current_window_reverse_complement_sequence=reverse($transition_sequence);
                                        
                                        
                                        
                                        #500B window call of PRE_predict sub
                                        ######################################################
                                        ######################################################
                                        ($hit_found_returned,$cluster_start_returned,$cluster_stop_returned,$PRE_id_returned,$cluster_length_returned)=PRE_predict (
                                                                                                                                                                                $sequenceTitle,
                                                                                                                                                                                $start_500B,
                                                                                                                                                                                $end,
                                                                                                                                                                                $current_window_sequence,
                                                                                                                                                                                $current_window_reverse_complement_sequence,
                                                                                                                                                                                $current_seq_length,
                                                                                                                                                                                $first_cluster_start,
                                                                                                                                                                                $first_cluster_stop,
                                                                                                                                                                                $PRE_id_start,
                                                                                                                                                                                $first_cluster_length
                                                                                                                                                                             );
                                        
                                        
                                            $hit_found=$hit_found_returned;
                                                
                                            if($hit_found==1)
                                                
                                            {
                                                
                                                    
                                                    $first_cluster_start=$cluster_start_returned;
                                                    $first_cluster_stop=$cluster_stop_returned;			
                                                    $PRE_id_start=$PRE_id_returned;
                                                    $first_cluster_length=$cluster_length_returned;
                                                    
                                            }
                                        
                                        
                                    }#end of 500B sub window	
           
            
            
            
            
            
            
                                            #start of 250A sub window
                                            #this part is evaluvated if 750, 500A and 500B fails to detect any boudary, $start is same as 750, end is modified as $start+250
                                            if ($hit_found==0) 
                                                
                                            {
                                                
                                                
                                                #to get the end postion of first 500bp out of 750 bp window, start remains same
                                                my$end_250A=$start+250;
                                                #to limit the end sequence to the maximum seq length
                                                    
                                                
                                                if($end_250A>=$sequenceLength)
                                                {$end_250A=$sequenceLength;}
                                                
                                                
                                                #to get current seq
                                                $current_window_sequence = substr($sequenceEntry, $start, $end_250A - $start);
                                                $current_seq_length = length($current_window_sequence);
                                                
                                                
                                                
                                                #to get the reverse complement of current seq
                                                #to assigin the $current_window_sequence to $transition_sequence
                                                $transition_sequence=$current_window_sequence;
                                                
                                                
                                                #complement the sequence
                                                $transition_sequence=~ tr/gatcryswkmbdhvnGATCRYSWKMBDHVN/ctagyrswmkvhdbnCTAGYRSWMKVHDBN/;
                                                
                                                #reverse the sequence to get the reverse complement
                                                $current_window_reverse_complement_sequence=reverse($transition_sequence);
                                                    
                                                    
                                                    
                                                #250A window call of PRE_predict sub
                                                ######################################################
                                                ######################################################
                                                ($hit_found_returned,$cluster_start_returned,$cluster_stop_returned,$PRE_id_returned,$cluster_length_returned)=PRE_predict (
                                                                                                                                                                                        $sequenceTitle,
                                                                                                                                                                                        $start,
                                                                                                                                                                                        $end_250A,
                                                                                                                                                                                        $current_window_sequence,
                                                                                                                                                                                        $current_window_reverse_complement_sequence,
                                                                                                                                                                                        $current_seq_length,
                                                                                                                                                                                        $first_cluster_start,
                                                                                                                                                                                        $first_cluster_stop,
                                                                                                                                                                                        $PRE_id_start,
                                                                                                                                                                                        $first_cluster_length
                                                                                                                                                                                     );
                                                    
                                                    
                                                    $hit_found=$hit_found_returned;
                                                    
                                                    if($hit_found==1)
                                                    
                                                        {
                                                            
                                                            
                                                            $first_cluster_start=$cluster_start_returned;
                                                            $first_cluster_stop=$cluster_stop_returned;			
                                                            $PRE_id_start=$PRE_id_returned;
                                                            $first_cluster_length=$cluster_length_returned;
                                                            
                                                            
                                                        }
                                                        
                                                        
                                            }#end of 250A sub window	
                            
                            
                            
                            
                            
                                                    #start of 250B sub window
                                                    #this part is evaluvated if 750, 500A, 500B and 250A fails to detect any boudary, start is changed as $start+250, end is $start+500
                                                    if (($hit_found==0) && ($current_seq_length_in_full_750bp_window>=250))
                                                    
                                                    {
                                                        #to get the start postion of second 250bp out of 750 bp window, start=start+250, the end is start+500
                                                        my$start_250B=$start+250;
                                                        
                                                        #to limit the end sequence to the maximum seq length
                                                        
                                                        my$end_250B=$start+500;
                                                        
                                                        if($end_250B>=$sequenceLength)
                                                        {$end_250B=$sequenceLength;}
                                                        
                                                           
                                                        #to get current seq
                                                        $current_window_sequence = substr($sequenceEntry, $start_250B, $end_250B - $start_250B);
                                                        $current_seq_length = length($current_window_sequence);
                                                        
                                                        
                                                        #to get the reverse complement of current seq
                                                        #to assigin the $current_window_sequence to $transition_sequence
                                                        $transition_sequence=$current_window_sequence;
                                                        
                                                        
                                                        #complement the sequence
                                                        $transition_sequence=~ tr/gatcryswkmbdhvnGATCRYSWKMBDHVN/ctagyrswmkvhdbnCTAGYRSWMKVHDBN/;
                                                        
                                                        
                                                        #reverse the sequence to get the reverse complement
                                                        $current_window_reverse_complement_sequence=reverse($transition_sequence);
                                                          
                                                        
                                                        #250B window call of PRE_predict sub
                                                        ######################################################
                                                        ######################################################    
                                                        ($hit_found_returned,$cluster_start_returned,$cluster_stop_returned,$PRE_id_returned,$cluster_length_returned)=PRE_predict (
                                                                                                                                                                                                $sequenceTitle,
                                                                                                                                                                                                $start_250B,
                                                                                                                                                                                                $end_250B,
                                                                                                                                                                                                $current_window_sequence,
                                                                                                                                                                                                $current_window_reverse_complement_sequence,
                                                                                                                                                                                                $current_seq_length,
                                                                                                                                                                                                $first_cluster_start,
                                                                                                                                                                                                $first_cluster_stop,
                                                                                                                                                                                                $PRE_id_start,
                                                                                                                                                                                                $first_cluster_length
                                                                                                                                                                                             );
                                                            
                                                            
                                                            $hit_found=$hit_found_returned;
                                                                
                                                                if($hit_found==1)
                                                                
                                                                {
                                                                    
                                                                    
                                                                    $first_cluster_start=$cluster_start_returned;
                                                                    $first_cluster_stop=$cluster_stop_returned;			
                                                                    $PRE_id_start=$PRE_id_returned;
                                                                    $first_cluster_length=$cluster_length_returned;
                                                                  
                                                                    
                                                                }
                                                        
                                                        
                                                    }#end of 250B sub window	                           
                            
                            
                            
                            
                                                            #start of 250C sub window
                                                            #this part is evaluvated if 750, 500A, 500B, 250A and 250B fails to detect any boudary, start is changed as $start+500, end is same as 750
                                                             if (($hit_found==0) && ($current_seq_length_in_full_750bp_window>=500))
                                                               
                                                            {
                                                                
                                                                #to get the start postion of third 250bp out of 750 bp window, start=start+500, the end is same as main window 
                                                                 my$start_250C=$start+500;
                                                                #to limit the end sequence to the maximum seq length
                                                                    
                                                                #to get current seq
                                                                $current_window_sequence = substr($sequenceEntry, $start_250C, $end - $start_250C);
                                                                $current_seq_length = length($current_window_sequence);
                                                                
                                                                #to get the reverse complement of current seq
                                                                #to assigin the $current_window_sequence to $transition_sequence
                                                                $transition_sequence=$current_window_sequence;
                                                                
                                                                
                                                                #complement the sequence
                                                                $transition_sequence=~ tr/gatcryswkmbdhvnGATCRYSWKMBDHVN/ctagyrswmkvhdbnCTAGYRSWMKVHDBN/;
                                                                
                                                                
                                                                #reverse the sequence to get the reverse complement
                                                                $current_window_reverse_complement_sequence=reverse($transition_sequence);
                                                                
                                                                
                                                                
                                                                #250C window call of PRE_predict sub
                                                                ######################################################
                                                                ######################################################    
                                                                ($hit_found_returned,$cluster_start_returned,$cluster_stop_returned,$PRE_id_returned,$cluster_length_returned)=PRE_predict (
                                                                                                                                                                                                        $sequenceTitle,
                                                                                                                                                                                                        $start_250C,
                                                                                                                                                                                                        $end,
                                                                                                                                                                                                        $current_window_sequence,
                                                                                                                                                                                                        $current_window_reverse_complement_sequence,
                                                                                                                                                                                                        $current_seq_length,
                                                                                                                                                                                                        $first_cluster_start,
                                                                                                                                                                                                        $first_cluster_stop,
                                                                                                                                                                                                        $PRE_id_start,
                                                                                                                                                                                                        $first_cluster_length
                                                                                                                                                                                                     );
                                                              
                                                                
                                                                $hit_found=$hit_found_returned;
                                                                
                                                                if($hit_found==1)
                                                                
                                                                {
                                                                    
                                                                    
                                                                    $first_cluster_start=$cluster_start_returned;
                                                                    $first_cluster_stop=$cluster_stop_returned;			
                                                                    $PRE_id_start=$PRE_id_returned;
                                                                    $first_cluster_length=$cluster_length_returned;
                                                                    
                                                                    
                                                                }
                                                                
                                                                
                                                            }#end of 250C sub window	                    
                                
                                
        
                
                
                
                
                
                
            # to move the window sequence using window increment value
            $start=$start+$window_increment;
                
                
                
                
        }#end of while loop that performs prediction to the given sequence			      
			      
			      
            
            
        #clear the sequence from the memory   
        #$sequenceEntry='';    
            
            
            
            
            
        
        
        
        # if no hit is found
        #######################################################
        #######################################################
        #######################################################
        if ($PRE_id_start==0)
        {
            #########################to print the summary_table_file#################################################################################
            open(OUTFILE, "+>>" . $summary_file_with_path) or die ("Cannot open file for output: $!");			      
            print(OUTFILE "$sequenceTitle\t$sequenceLength\t0\n");
            close(OUTFILE) or die ("Cannot close file for output: $!");
            ##########################################################################End of print ##################################################
            #print"\nNo PRE hit found for the given input sequence.\n";
                           
        }
        
        
        # if a or more hit is found, then proces the hits and create output files
        #######################################################
        #######################################################
        #######################################################
        else
        {
            # to sort the all_hits_table_file
            #################################
            #################################
            my$sorting_input_file=$hits_table_file_name_with_path;
            
            
            
            
            
            
            
            
            
            
            
            #call a sub to process overlaping hits table and get an array of merged PREs as return
            ##################################################
            ##################################################
            ##################################################
            my @received_merged_PREs=();
            @received_merged_PREs=sort_hitstable_remove_duplicates_find_distance_and_merge_overlapping_hits($sorting_input_file);
            #print"\nYYYYYYYYYYY@received_merged_PREs\n";
            #end of call for sub 
            ##################################################
            ##################################################
            ##################################################
            
            
            
            
            
            
            
            
            
            #call a sub to repredict_PREs_for_merged_regions
            ##################################################
            ##################################################
            ##################################################
            re_predict_PREs_for_merged_regions(@received_merged_PREs);
            #end of call for sub repredict_PREs_for_merged_regions
            ##################################################
            ##################################################
            ##################################################
            
            
            

            
            
            
            
            
            #to run merge function for merging overlaping hits
            ##################################################
            ##################################################
            #merge ($input_fasta_file_name,$input_genbank_file_name,$table_output_file_name,$detailed_output_file_name,$sequence_out_file_name,$graphical_output_file_name,$sequenceLength,$scale_for_graphical_output,$image_width);
            
            
            
            
               
        }# end of if hit is found loop
        



##################################################################### subroutines ###################################################################
#####################################################################################################################################################
#####################################################################################################################################################


#sub to predict PRE in given sub sequence
##############################################
##############################################
sub PRE_predict  
    {
        
                
        
                                                                                                                                                              
        
        
        
		my$sequenceTitle_value=shift;
        my$start_value=shift;
        my$end_value=shift;
        my$prediction_sequence=shift;
        my$prediction_sequence_reverse_complement=shift;
        my$prediction_sequence_length=shift;
        my$first_cluster_start_value=shift;
        my$first_cluster_stop_value=shift;
        my$PRE_id_value=shift;
        my$first_cluster_length_value=shift;
        
        
  
        
        my$hit_found=0;
        
        
        
        #initialization of variables used within this function
        my$calculated_average_gap='';
        my$dif_kind_motif=0;
        my$first_motif_start_line='';
        my$second_motif_start_line='';
        my$motif_name1='';
        my$motif_name2='';
        my$motif_gap='';
        my$cluster_start=0;
        my$cluster_stop=0;
        my$cluster_length=0;
                        
                    
		    
                    
		    #for DSP1
		    my$DSP1_found=0;
			
			#for GAGA 
		    my$gaga_found=0;
                    
		    #for Zeste
		    my$Zeste_found=0;
                   
		    #for Sp1
		    my$Sp1_found=0;
		    
		    #for pho
		    my$Pho_found=0;
		    
			#for EN1
		    my$EN1_found=0;
			
			
			#for ADF1
			my$ADF1_found=0;
				
			#for Combgap
			my$Combgap_found=0;
			
			
			#for Grh site 1
			my$grh_1_found=0;
			
			#for Grh site 2
			my$grh_2_found=0;
			


			
			my$Pho_overlapping_site=0;	
            my$total_SuHw_found=0;
                    

                    
		    
		    
		    
                    #for total number of motifs with out overlapping Su(Hw) sites
		    my$total_number_of_motifs_after_removing_overlapping=0;
			
		    #An array to hold results
		    my @resultsArray=();
		    my@startArray=();
                    my@stopArray=();
                    
                    
                    
                    
                    
		    #The pattern to match											       #Y  R    Y TGCATA Y   Y   Y    # Y  W  GC M  TACTT H     Y
		    my @PRE_motifs = ("GAAAA","GAGAG","[C/T]GAG[C/T]G","[G/A][G/A]GG[T/C]G","C[A/T/G/C]GCCAT[A/T/G/C][A/T/G][A/T/G/C][A/T/G/C][A/T/G]","G[C/G][A/T/G/C][A/C]ACGCCCC","G[C/G][A/T/G/C][A/C]ACGCCCC","[G/T]C[G/A][G/A]C[G/A]GC[G/A][G/A]C[G/A]","GTGTGT","TGTTTTTT", "[A/T]C[A/T/C]GGTT");
					
                        
            foreach my$PRE_motifs(@PRE_motifs) 
            {
                #Removes the first element in the array and assign it to pattern variable
                my $pattern = $PRE_motifs;
                        
                my $strandSearched;
                my $matchingSequence;
                my $firstMatchingPosition;
                my $lastMatchingPosition;
                my $firstMatchingPosition1;
                my $lastMatchingPosition1;
                my $firstMatchingPosition2;
                my $lastMatchingPosition2;
                        
                my$motif_name;
			
                        
                #Perform the search for the direct strand first.
                        
                $strandSearched = "+";
                
                  
                while ($prediction_sequence =~ m/($pattern)/gi) 
                {
                    
    			    $matchingSequence = $1;
    			    $lastMatchingPosition = pos($prediction_sequence);
    			    
                    $firstMatchingPosition = $lastMatchingPosition - length($matchingSequence)+ 1;
                            
                    # count for no motifs
                    if ($pattern eq "GAAAA")
    			    {$DSP1_found=$DSP1_found+1;$motif_name="DSP1";}
                            
                    elsif ($pattern eq "GAGAG")
    			    {$gaga_found=$gaga_found+1;$motif_name="GAGA";}
                            
                    elsif ($pattern eq "[C/T]GAG[C/T]G")
    			    {$Zeste_found=$Zeste_found+1;$motif_name="Zeste";}
                            
                    elsif ($pattern eq "[G/A][G/A]GG[T/C]G")
    			    {$Sp1_found=$Sp1_found+1;$motif_name="Sp1";}
                            
                    elsif ($pattern eq "C[A/T/G/C]GCCAT[A/T/G/C][A/T/G][A/T/G/C][A/T/G/C][A/T/G]")
    			    {$Pho_found=$Pho_found+1;$motif_name="Pho";}
                    
                    elsif ($pattern eq "G[C/G][A/T/G/C][A/C]ACGCCCC")
    			    {$EN1_found=$EN1_found+1;$motif_name="EN1";}
                    
                    
					elsif ($pattern eq "[G/T]C[G/A][G/A]C[G/A]GC[G/A][G/A]C[G/A]")
                    {$ADF1_found=$ADF1_found+1;$motif_name="ADF1";}		
				
					elsif ($pattern eq "GTGTGT")
                    {$Combgap_found=$Combgap_found+1;$motif_name="Combgap";}		
                    
					
					elsif ($pattern eq "TGTTTTTT")
                    {$grh_1_found=$grh_1_found+1;$motif_name="Grh-1";}
					
					elsif ($pattern eq "[A/T]C[A/T/C]GGTT")
                    {$grh_2_found=$grh_2_found+1;$motif_name="Grh-2";}
					
					
					
					
                    $firstMatchingPosition1=$firstMatchingPosition+$start_value;
                    $lastMatchingPosition1=$lastMatchingPosition+$start_value;
                            
                            
                    push( @resultsArray, "$motif_name\t\t$firstMatchingPosition1\t$lastMatchingPosition1\t$strandSearched\t$matchingSequence\t\t");   
                            
                    push( @startArray, "$firstMatchingPosition1\t"); 
                    push( @stopArray, "$lastMatchingPosition1\t");
                            
                }#end of while loop for each + strand match
                        
			
			
                # to search in - strand
                $strandSearched = "-";
                            
                            
                            
                while ($prediction_sequence_reverse_complement =~ m/($pattern)/gi)               
                {
                            
                    $matchingSequence = $1;
                            
                    #These calculations make the positions correspond to the direct strand.
    			    $firstMatchingPosition = $prediction_sequence_length - pos($prediction_sequence_reverse_complement) + 1;
    			    $lastMatchingPosition = $firstMatchingPosition + length($matchingSequence)- 1;
                    
                    # count for no motifs
                    if ($pattern eq "GAAAA")
    			    {$DSP1_found=$DSP1_found+1;$motif_name="DSP1";}
                          
                    elsif ($pattern eq "GAGAG")
    			    {$gaga_found=$gaga_found+1;$motif_name="GAGA";}
                            
                    elsif ($pattern eq "[C/T]GAG[C/T]G")
    			    {$Zeste_found=$Zeste_found+1;$motif_name="Zeste";}
                            
                    elsif ($pattern eq "[G/A][G/A]GG[T/C]G")
    			    {$Sp1_found=$Sp1_found+1;$motif_name="Sp1";}
                            
                    elsif ($pattern eq "C[A/T/G/C]GCCAT[A/T/G/C][A/T/G][A/T/G/C][A/T/G/C][A/T/G]")
    			    {$Pho_found=$Pho_found+1;$motif_name="Pho";}
                            
                    elsif ($pattern eq "G[C/G][A/T/G/C][A/C]ACGCCCC")
    			    {$EN1_found=$EN1_found+1;$motif_name="EN1";}				
                            
                    
					elsif ($pattern eq "[G/T]C[G/A][G/A]C[G/A]GC[G/A][G/A]C[G/A]")
                    {$ADF1_found=$ADF1_found+1;$motif_name="ADF1";}		
				
					elsif ($pattern eq "GTGTGT")
                    {$Combgap_found=$Combgap_found+1;$motif_name="Combgap";}
					
					
					elsif ($pattern eq "TGTTTTTT")
                    {$grh_1_found=$grh_1_found+1;$motif_name="Grh-1";}
					
					elsif ($pattern eq "[A/T]C[A/T/C]GGTT")
                    {$grh_2_found=$grh_2_found+1;$motif_name="Grh-2";}
					
				
					
					
                    $firstMatchingPosition2=$firstMatchingPosition+$start_value;
                    $lastMatchingPosition2=$lastMatchingPosition+$start_value;
                            
                    
                    push( @resultsArray, "$motif_name\t\t$firstMatchingPosition2\t$lastMatchingPosition2\t$strandSearched\t$matchingSequence\t\t"); 
                              
                    push( @startArray, "$firstMatchingPosition2\t"); 
                    push( @stopArray, "$lastMatchingPosition2\t"); 
                    
                            
                         
                          
                         
                          
                }#end of while loop for - strand each match
                        
                        
                        
                        
            }#end of for each motif search loop 			      
                  
                  
                   
		    #Sort the results by start base using a map-sort-map.
		    @resultsArray = map { $_->[1] }
      		sort { $a->[0] <=> $b->[0]}
      		map { [ getStartBase($_), $_] }
      		@resultsArray;
                    
		    #sub used for sorting
            sub getStartBase 
		    {
                        
                        my $result = shift();
                        $result =~ m/^\S+\t\t(\d+)\t\d+\t[\+\-]\t\S+/;
                        return $1;
                        
                        
            }
		    
                    
                    
                    
                    
                    
            #Sort the results by start base using a map-sort-map.
		    @startArray = map { $_->[1] }
      		sort { $a->[0] <=> $b->[0]}
      		map { [ getStartBase1($_), $_] }
      		@startArray;
                    
                    
            #print"@startArray\n";
                    
		    #sub used for sorting
            sub getStartBase1    
		    {
                        
                my $result = shift();
                $result =~ m/^(\d+)\t/;
                return $1;
                
			}
                    
                    
            #Sort the results by start base using a map-sort-map.
		    @stopArray = map { $_->[1] }
      		sort { $a->[0] <=> $b->[0]}
      		map { [ getStartBase2($_), $_] }
      		@stopArray;
                    
                    
		    #sub used for sorting
            sub getStartBase2     
		    {
                        
                my $result = shift();
                $result =~ m/^(\d+)\t/;
                return $1;
                
            }
                    
                    
                    
                    
		    my$numberOfMatches=0;
		    # to find no of element in the array
		    $numberOfMatches = scalar(@resultsArray);
		    
                    
                    
            # proceed further if no of motifs are more than two else return default values
            if($numberOfMatches>=2)
		    {
                
                # to find the cluster start and end 
                my$cluster_start_line=$resultsArray[0];
                my$cluster_end_line=$resultsArray[-1];
			
			
                if ($cluster_start_line =~ m/^\S+\t\t(\d+)\t\d+\t[\+\-]\t\S+/)
			
                {$cluster_start=$1;}
			
			
                if ($cluster_end_line =~ m/^\S+\t\t\d+\t(\d+)\t[\+\-]\t\S+/)
			
                {$cluster_stop=$1;}
			
                $cluster_length=$cluster_stop-$cluster_start+1;			
			
                        
                        
                # to find the cluster start and end 
                my$new_cluster_start=0;
                my$new_cluster_stop=0;
                        
                my$new_cluster_start_line=$startArray[0];
                if ($new_cluster_start_line =~ m/^(\d+)\t/)
                {$new_cluster_start=$1;}
                        
                my$new_cluster_end_line=$stopArray[-1];
                if ($new_cluster_end_line =~ m/^(\d+)\t/)
                {$new_cluster_stop=$1;}
                        
                        
                my$new_cluster_length=$new_cluster_stop-$new_cluster_start+1;
                        
                        
                        
                        
                        
                        
                        
                        
                        
                        
                        
                        
                       
			my$potential_Zeste_binding_sites=0;		
			my$potential_DSP1_binding_sites=0;
			# to add the gap detail to the final file 
                        
			my@resultsArray1=();	
                        
			#to push the first element in the array with 0 gap
			push( @resultsArray1, "$cluster_start_line\t0\n"); 
                        
                      
			#to find the gap between each motif and write it to new array 
			#############################################################
            #############################################################
            for (my $i = 0 ; $i <= $numberOfMatches-2; $i++ )                 
			{
                            
			    my$current_first_motif_stop=0;
			    my$current_second_motif_start=0;
                            
			    # to find the cluster start and end
			 
			    $first_motif_start_line=$resultsArray[$i];
			    $second_motif_start_line=$resultsArray[$i+1];
                            
                if ($first_motif_start_line =~ m/^(\S+)\t\t\d+\t(\d+)\t[\+\-]\t\S+/)   
				{
                                    
                    $motif_name1=$1;
                    $current_first_motif_stop=$2;
                                    
				}
                                
                              
                                
			    if ($second_motif_start_line =~ m/^(\S+)\t\t(\d+)\t\d+\t[\+\-]\t\S+/)                
				{
                                    
                    $motif_name2=$1;
                    $current_second_motif_start=$2;
                                    
                }
                              
                              
                              
                $motif_gap=$current_second_motif_start-$current_first_motif_stop;
                                    
                                    
                
                          
                #to push the remaining elements in the array with gap
                push( @resultsArray1, "$second_motif_start_line\t$motif_gap\n"); 
                            
                          
                           
            }# closing of writing to new array	
                        
                        
			
			
			
                        
                        
                        
                        
			if($DSP1_found>=1)
			{$dif_kind_motif=$dif_kind_motif+1;}
			
                        
			if($gaga_found>=1)
			{$dif_kind_motif=$dif_kind_motif+1;}            
			
			
			if($Zeste_found>=1)
			{$dif_kind_motif=$dif_kind_motif+1;}
			
			
			if($Sp1_found>=1)
			{$dif_kind_motif=$dif_kind_motif+1;}
			
            
			if($Pho_found>=1)
			{$dif_kind_motif=$dif_kind_motif+1;}
			
			
			if($EN1_found>=1)
			{$dif_kind_motif=$dif_kind_motif+1;}
			
			
			if($ADF1_found>=1)
			{$dif_kind_motif=$dif_kind_motif+1;}
		
		
			if($Combgap_found>=1)
			{$dif_kind_motif=$dif_kind_motif+1;}
			
			if(($grh_1_found>=1) or ($grh_2_found>=1))
			{$dif_kind_motif=$dif_kind_motif+1;}
			
			
			
			# to calculate the score
			my$cluster_score=0;
			
                        $calculated_average_gap=($cluster_length/$numberOfMatches);
                        $calculated_average_gap= sprintf("%.2f", $calculated_average_gap);
                            
                            
                        #intitiation of score values for each motif
                        
                        my$gaga_score=0;
                        my$DSP1_score=0;
                        my$Zeste_score=0;
                        my$Sp1_score=0;
                        my$Pho_score=0;
                        my$EN1_score=0;
                        my$ADF1_score=0;		
						my$Combgap_score=0;
						my$grh_1_score=0;
						my$grh_2_score=0;
						my$grh_3_score=0;
                           
                        #multiplication of scores for each motif and their occurences
                          
                        $gaga_score=$gaga_found*1;
                        $DSP1_score=$DSP1_found*1;
                        $Zeste_score=$Zeste_found*1;
                        $Sp1_score=$Sp1_found*1;
                        $Pho_score=$Pho_found*1;
                        $EN1_score=$EN1_found*1;
						$ADF1_score=$ADF1_found*1;	
						$Combgap_score=$Combgap_found*1;
						$grh_1_score=$grh_1_found*1;
						$grh_2_score=$grh_2_found*1;
						
						
						
                        # to find the total no of motifs
                        $total_number_of_motifs_after_removing_overlapping=$numberOfMatches;			
                        
                        
                        # the final score
                        $cluster_score=$gaga_score+$DSP1_score+$Zeste_score+$Sp1_score+$Pho_score+$EN1_score+$ADF1_score+$Combgap_score+$grh_1_score+$grh_2_score+$grh_3_score;
                        $cluster_score= sprintf("%.2f", $cluster_score);
                      
                     
                        
			
                        
                            ##################################################################
                            
                            
                            #print results if it is not a duplicate
                            
                            if(($total_number_of_motifs_after_removing_overlapping>=10)&&($first_cluster_start_value!=$cluster_start)&&($first_cluster_length_value!=$cluster_length))  
                            {
                                
                                
                                ##############################################################
                                ##############################################################                             
                                if($Pho_found>=1 && $gaga_found>=1 && $DSP1_found>=1 && $Zeste_found>=1 && $Sp1_found>=1 && $calculated_average_gap<=60 && $total_number_of_motifs_after_removing_overlapping>=10)    
                                {	
                                    
                                    $PRE_id_value=$PRE_id_value+1;
                                    
                                    
                                    open(OUTFILE, "+>>" . $hits_table_file_name_with_path) or die ("Cannot open file for output: $!");
                                    print(OUTFILE "PRE\t$PRE_id_value\t$cluster_start\t$cluster_stop\t$cluster_length\t$calculated_average_gap\t$cluster_score\t$total_number_of_motifs_after_removing_overlapping\n");
                                    close(OUTFILE) or die ("Cannot close file for output: $!");
                                    #end of printing for a given seq
                                    
                                    
                                    $hit_found=1;
                                    
                                    #to return the values to main program
                                    return ($hit_found,$cluster_start,$cluster_stop,$PRE_id_value,$cluster_length);
                                    
                                }#close of printing
								
								elsif($Pho_found>=1 && $dif_kind_motif>=4 && $calculated_average_gap<=40 && $total_number_of_motifs_after_removing_overlapping>=16)    
                                {	
                                    
                                    $PRE_id_value=$PRE_id_value+1;
                                    
                                    
                                    open(OUTFILE, "+>>" . $hits_table_file_name_with_path) or die ("Cannot open file for output: $!");
                                    print(OUTFILE "PRE\t$PRE_id_value\t$cluster_start\t$cluster_stop\t$cluster_length\t$calculated_average_gap\t$cluster_score\t$total_number_of_motifs_after_removing_overlapping\n");
                                    close(OUTFILE) or die ("Cannot close file for output: $!");
                                    #end of printing for a given seq
                                    
                                    
                                    $hit_found=1;
                                    
                                    #to return the values to main program
                                    return ($hit_found,$cluster_start,$cluster_stop,$PRE_id_value,$cluster_length);
                                    
                                }#close of printing
								
								elsif($Combgap_found>=2 && $dif_kind_motif>=4 && $calculated_average_gap<=40 && $total_number_of_motifs_after_removing_overlapping>=16)    
                                {
									
                                    
                                    $PRE_id_value=$PRE_id_value+1;
                                    
                                    
                                    open(OUTFILE, "+>>" . $hits_table_file_name_with_path) or die ("Cannot open file for output: $!");
                                    print(OUTFILE "PRE\t$PRE_id_value\t$cluster_start\t$cluster_stop\t$cluster_length\t$calculated_average_gap\t$cluster_score\t$total_number_of_motifs_after_removing_overlapping\n");
                                    close(OUTFILE) or die ("Cannot close file for output: $!");
                                    #end of printing for a given seq
                                    
                                    
                                    $hit_found=1;
                                    
                                    #to return the values to main program
                                    return ($hit_found,$cluster_start,$cluster_stop,$PRE_id_value,$cluster_length);
                                    
                                }#close of printing
								
								
                                else
                                {
                                    
                                    
                                    #to return the values to main program
                                    return ($hit_found,$cluster_start,$cluster_stop,$PRE_id_value,$cluster_length);
                                    
                                    
                                }#close of else  
                                
                                
                                
                                
                               
                                
                                
                                
                            }#close of duplicate if loop check    
                            
                            
                         
                        
                        
                
                   
    }#to return the values to main program
            else{return ($hit_found,$cluster_start,$cluster_stop,$PRE_id_value,$cluster_length);}
     
            
        
    }#end of sub_prection function 			      

#end of sub_prection function 
##############################################
##############################################
##############################################




































#start of sub sort_hitstable_remove_duplicates_find_distance_and_merge_overlapping_hits
##############################################
##############################################
##############################################
sub sort_hitstable_remove_duplicates_find_distance_and_merge_overlapping_hits
    {
       
        my$c_file_name = shift;
        
        #print"$c_file_name\nNNNNNNNNN";
        #to initiate the overlapping_unique_hits.txt output file
        open(OUTFILE, ">" . $overlapping_unique_hits_file_name_with_path) or die ("Cannot open file for output: $!");
        print(OUTFILE "PRE\tStart\tStop (Size)\tDistance (kb)\n");
        close(OUTFILE) or die ("Cannot close file for output: $!");
        

        
        #empty array intiation to hold hits_table_file
        my @ht_file = (  );
        
        
        #empty array intiation to hold sorted hits_table_file
        my @sorted_ht = (  );
        
        
        #variables used for sorting the list
        my$PRE_start= '';
        my$PRE_stop= '';
        my$PRE_size= '';
        my$PRE_avg_gap= '';
        my$PRE_score= '';
        my$total_no_of_motifs= '';
        



        #open(FILE, $c_file_name) || die "Can't open $c_file_name: $!\n";
        #while (<FILE>) {
                 #if($_ =~ m/^PRE/){print"$_";push(@ht_file, $_)};
                
                
        #}
        # reading file data using an array
        @ht_file = get_file_data1($c_file_name);
        #print"@ht_file\nMMMMMMMMM";
        
        #close(FILE);      
        
        my$scaler_of_ht_file=scalar(@ht_file);

        #print"\n$scaler_of_ht_file\n";print"\n$ht_file[0]\n";
        #print"Array\t@ht_file\n";
        #Starting of sorting hits tabe
        #############################################################################################################################
        #############################################################################################################################
        #############################################################################################################################
        #sorting the list and store it in a array
        foreach my$line (@ht_file) 
	{
            
            while($line =~ m/PRE\t\d+\t(\d+)\t(\d+)\t(\d+)\t(\d+\.?\d+)\t(\d+\.?\d+)\t(\d+)/gi)
            {
    		$PRE_start =$1;
		$PRE_stop =$2;
		$PRE_size =$3;
		$PRE_avg_gap=$4;
		$PRE_score=$5;
		$total_no_of_motifs=$6;
		
		
		#assign to an array
		push( @sorted_ht, "$PRE_start\t$PRE_stop\t$PRE_size\t$PRE_avg_gap\t$PRE_score\t$total_no_of_motifs\n");					
                
                #sort the array by start base using a map-sort-map.
                @sorted_ht= map { $_->[1] }
                sort { $a->[0] <=> $b->[0]}
                map { [ getStartBase3($_), $_] }
                @sorted_ht;
                
                sub getStartBase3 
                {
                    my $result = shift();
                    $result =~ m/^(\d+)\t\d+\t\d+\t\d+\.?\d+\t\d+\.?\d+\t\d+/;
                    return $1;
                }
                
            }#end of if loop
            
            
            
        }#end of foreach loop
        #End of sorting hits tabe
        #############################################################################################################################
        #############################################################################################################################
        #############################################################################################################################
        
        
        
        
        
        #Starting of duplicate removal
        #############################################################################################################################
        #############################################################################################################################
        #############################################################################################################################
        #variables used for duplicate removal
        my$PRE_start1= '';
        my$PRE_stop1= '';
        my$PRE_size1= '';
        my$PRE_avg_gap1= '';
        my$PRE_score1= '';
        my$total_no_of_motifs1= '';
        
        my$PRE_start2= '';
        my$PRE_stop2= '';
        
        #initiate a empty array
        my @unique_PREs = (  );
        
        #assign the first value of duplecate containing list to a variable and push as a first element in unique array to avoid array being empty
        my$first_line=$sorted_ht [0];
        push(@unique_PREs, $first_line); 
        
        my$number_hits_in_unique_bounadry= 0;                              
        
        
        
        #removal of duplicates from the list
        foreach my$line1 (@sorted_ht) 
	{
            my$flag=1;
            if($line1 =~ /^(\d+)\t(\d+)\t(\d+)\t(\d+\.?\d+)\t(\d+\.?\d+)\t(\d+)/) 
            {
                $PRE_start1 =$1;
		$PRE_stop1 =$2;
                $PRE_size1 =$3;
		$PRE_avg_gap1=$4;
		$PRE_score1=$5;
                $total_no_of_motifs1=$6;
                
                # to find no of element in the unique PREs array
                $number_hits_in_unique_bounadry = scalar(@unique_PREs);
                
                #perform a for loop for unique PREs array elements
                for (my $i = 0 ; $i < $number_hits_in_unique_bounadry; $i++ ) 
		{
                    if ($unique_PREs[$i] =~ m/^(\d+)\t(\d+)\t\d+\t\d+\.?\d+\t\d+\.?\d+\t\d+/)       
                    {
			$PRE_start2=$1;
			$PRE_stop2=$2;
                    }        
			
                    #compare the values in the non-unique array with unique array values if it matches set the flag value as 0  
                    if(($PRE_start1==$PRE_start2) &&($PRE_stop1==$PRE_stop2))
                    {
			$flag=0;
			
                    }
                    
                }#end of foreach loop
					
					
					
                                        
                                        
                #if it is unique(flag value is 1) then add to unique array else don't add.
                if($flag==1)
                {
                    push(@unique_PREs, "$PRE_start1\t$PRE_stop1\t$PRE_size1\t$PRE_avg_gap1\t$PRE_score1\t$total_no_of_motifs1\n"); 
		}
		
                
                
            }#end of if loop
            
            
            
        }#end of foreach loop
        #End of duplicate removal
        #############################################################################################################################
        #############################################################################################################################
        #############################################################################################################################
        
        
        
        
        
        
        
        
        
        
        
        #Starting of finding distance between PREs
        #############################################################################################################################
        #############################################################################################################################
        #############################################################################################################################
        ########to find the gap between PREs
        my$unique_PRE_start= '';
        my$unique_PRE_stop= '';
        my$unique_PRE_size= '';
        my$unique_PRE_avg_gap= '';
        my$unique_PRE_score= '';
        my$unique_PRE_total_no_of_motifs= '';
        my$unique_PRE_id= '';

        my$previous_unique_PRE_stop=0;
        # to find no of element in the array
        $number_hits_in_unique_bounadry = scalar(@unique_PREs);

        #to find the gap for each motif
        my$gap=0;
        
        for (my$i = 0; $i < $number_hits_in_unique_bounadry;$i++ )
        {
            if ($unique_PREs[$i] =~ m/^(\d+)\t(\d+)\t(\d+)\t(\d+\.?\d+)\t(\d+\.?\d+)\t(\d+)/)
            {
                $unique_PRE_start=$1;
		$unique_PRE_stop=$2;
		$unique_PRE_size=$3;
		$unique_PRE_avg_gap=$4;
		$unique_PRE_score=$5;
                $unique_PRE_total_no_of_motifs=$6;
                
                #to give PRE id
		$unique_PRE_id=$i+1;
                
                #to find the distance between PREs
		$gap=($unique_PRE_start-$previous_unique_PRE_stop);
								
		#to print in output file
		open(OUTFILE, "+>>" . $overlapping_unique_hits_file_name_with_path) or die ("Cannot open file for output: $!");
                print(OUTFILE "PRE\t$unique_PRE_id\t$unique_PRE_start\t$unique_PRE_stop\t$gap\n");
		close(OUTFILE) or die ("Cannot close file for output: $!");
								
		
		
            }  
            
            #to assign the current unique PRE stop to previous unique PRE stop
            $previous_unique_PRE_stop=$unique_PRE_stop;
            
        }#end of gap finding
        #End of finding distance between PREs
        #############################################################################################################################
        #############################################################################################################################
        #############################################################################################################################
        
        
       
        
        
        
        
        
        
        
    
        
        
        
        
        
        #Starting of merging overlapping PREs
        #############################################################################################################################
        #############################################################################################################################
        #############################################################################################################################
        
        #empty array intiation to hold overlapping_PREs_table_file
        my @ovl_PREs_file = (  );
        
        @ovl_PREs_file = get_file_data($overlapping_unique_hits_file_name_with_path);
        
	#print"\n#################\n@ovl_PREs_file\n";
        my $ovl_PRE_start= 0;
        my $ovl_PRE_stop= 0;
        my $ovl_PRE_distance= 1;
	
        my $ovl_PRE_start1= 0;
        my $ovl_PRE_stop1= 0;
	
        #empty array intiation to hold merged PREs
        my @merged_PREs = (  );
	
        #merge the overlapping hits and push it to an array called merged PREs
        foreach my$line (@ovl_PREs_file) 
        {
            #if($line =~ /^PRE\t*\d+\t*(\d+)\t*(\d*)\t*(\W\d+)/) 
            while($line =~ m/PRE\t*\d+\t(\d+)\t(\d+)\t(\-?\d+)/gi) 	
            {
                $ovl_PRE_start =$1;
                $ovl_PRE_stop =$2;
                $ovl_PRE_distance =$3;
                
                
                
                #if gap distance is equal or more than 1 base the call as separate hits
                if($ovl_PRE_distance>=1)
                {
                    #push it to merged array
                    push( @merged_PREs, "$ovl_PRE_start\t$ovl_PRE_stop\n");					
                    #to assign first the start value so that it can be used in case of overlapping hits
                    $ovl_PRE_start1=$ovl_PRE_start;
                    $ovl_PRE_stop1=$ovl_PRE_stop;
                    
                }  
             
                
                #else  the hits
                else
                {
                    #delete the last element in the merged array which contains correct start and old(false) stop value in case of overlapping hits
                    pop(@merged_PREs);
                    
                    if($ovl_PRE_stop>$ovl_PRE_stop1)
                    {
                        #replace the old stop value with new correct value
                        $ovl_PRE_stop1=$ovl_PRE_stop;
                    }
                    
                    #push it to merged array
                    push( @merged_PREs, "$ovl_PRE_start1\t$ovl_PRE_stop1\n");
                    
                }
                
                
                
                
            }#end of if loop
            
            #print"\n#################\n@merged_PREs\n";
            
        }#end of foreach loop		
        
        #End of merging overlapping PREs
        #############################################################################################################################
        #############################################################################################################################
        #############################################################################################################################
        
	
        
        
        #to return the merged_PREs_array to main
	return @merged_PREs;
        
        
        
        
       
    }#end of sub merge_hits_table
    
#end of sub sort_hitstable_remove_duplicates_find_distance_and_merge_overlapping_hits
##############################################
##############################################
##############################################










#start of sub re_predict_PREs_for_merged_regions
##############################################
##############################################
##############################################
sub re_predict_PREs_for_merged_regions
    {
        
        my @hits = @_;
        
       
        
        #Write header information to the file specified.
        open(OUTFILE, ">" . $table_output_file_name) or die ("Cannot open file for output: $!");
        print(OUTFILE "PRE\tStart-Stop\t\tSize\t\tTotal Motifs\tDistance (kb)\n");
        close(OUTFILE) or die ("Cannot close file for output: $!");
        
        
        #Write header information to the file specified.
	open(OUTFILE, ">" . $detailed_output_file_name) or die ("Cannot open file for output: $!");
	print(OUTFILE "**********************************************************************************\n");
	print(OUTFILE "******************************* Result of PRE Mapper Tool ************************\n");  
	print(OUTFILE "***********************AUTHORS: A.SRINIVASAN & RAKESH K MISHRA********************\n\n");
	print(OUTFILE "INPUT FILE USED: $input_fasta_file_name \n");
	print(OUTFILE "SEQUENCE TITLE: $sequenceTitle, SEQUENCE LENGTH: $sequenceLength.\n\n");
	close(OUTFILE) or die ("Cannot close file for output: $!");
        
        
	#to create sequence ouput file		
	open(OUTFILE, ">" . $sequence_out_file_name) or die ("Cannot open file for output: $!");
	close(OUTFILE) or die ("Cannot close file for output: $!");
        
        

        
        #to initiate the sequence output file
        open(OUTFILE, ">" . $table_to_graphics_output_file_name) or die ("Cannot open file for output: $!");
        close(OUTFILE) or die ("Cannot close file for output: $!");
        
        
        
        
        my $line_hits = '';
        my$merged_PRE_start= '';
        my$merged_PRE_stop= '';
        
        my$merged_PRE_start_actual=0;
        my$merged_PRE_stop_actual=0;
        
        my$cluster_start_print=0;
        my$cluster_stop_print=0;
        
        my$gap_to_print=1;
        
        my$extract=0;
        my$current_window_sequence='';
        my$current_seq_length=0;
        my$transition_sequence='';
        
        my$current_window_reverse_complement_sequence='';
        
        my $PRE_id=0;
        
        
        # for each merged hit perform PRE prediction
        foreach $line_hits (@hits) 
	{
            if($line_hits =~ /^(\d+)\t*(\d+)/) 
            {
    		$merged_PRE_start =$1;
		$merged_PRE_stop=$2;
		
                #minus 1 get the actual sequence (as numbers starts with 0 )
                $merged_PRE_start_actual =$merged_PRE_start-1;
		$merged_PRE_stop_actual=$merged_PRE_stop-1;
               
               
               
                $cluster_start_print=$merged_PRE_start;
                $gap_to_print=($cluster_start_print-$cluster_stop_print+1)/1000;
		$gap_to_print= sprintf("%.1f", $gap_to_print);
		
		$cluster_stop_print=$merged_PRE_stop;
		
                
                $extract=$merged_PRE_stop_actual-$merged_PRE_start_actual+1;
                
                #to get current seq
                $current_window_sequence = substr($sequenceEntry, $merged_PRE_start_actual, $extract);
                
                
                $current_seq_length = length($current_window_sequence);
                    
                #to assigin the $current_window_sequence to $transition_sequence
                $transition_sequence=$current_window_sequence;
                
                #complement the sequence
                $transition_sequence=~ tr/gatcryswkmbdhvnGATCRYSWKMBDHVN/ctagyrswmkvhdbnCTAGYRSWMKVHDBN/;
                
                #reverse the sequence to get the reverse complement
                $current_window_reverse_complement_sequence=reverse($transition_sequence);
               
                
                $PRE_id=$PRE_id+1;
                
               
                #call of re_predict_function to predict for a given region 
                re_predict ($sequenceTitle,$current_window_sequence,$current_window_reverse_complement_sequence,$current_seq_length,$PRE_id,$merged_PRE_start_actual,$merged_PRE_stop_actual,$gap_to_print,$table_output_file_name,$detailed_output_file_name,$sequence_out_file_name);
                
                
            }#end of if loop
            
            
	}#end of foreach loop
        
        
       	
        #########################to print the conditions used for running the program############################################################
        open(OUTFILE, "+>>" . $detailed_output_file_name) or die ("Cannot open file for output: $!");			      
        print(OUTFILE "\n\n\n");
        print(OUTFILE "****************************Conditions used prediction******************************\n\n");
        print(OUTFILE "Input fasta file used: $input_fasta_file_name\n");
        print(OUTFILE "sequence title used for prediction: $sequenceTitle\n");
        print(OUTFILE "Total no of hits found: $PRE_id\n\n");
        print(OUTFILE "************************************************************************************\n");
        close(OUTFILE) or die ("Cannot close file for output: $!");
        ##########################################################################End of print ##################################################			      
        
        my$positive_for_hit=0;
        
        if ($PRE_id>=0)
        {
            $positive_for_hit=1;
        }
        
        
        #########################to print the summary_table_file#################################################################################
        open(OUTFILE, "+>>" . $summary_file_with_path) or die ("Cannot open file for output: $!");			      
        print(OUTFILE "$sequenceTitle\t$sequenceLength\t$PRE_id\n");
        close(OUTFILE) or die ("Cannot close file for output: $!");
        ##########################################################################End of print ##################################################
        
        
        
        
    }
#end of sub re_predict_PREs_for_merged_regions
##############################################
##############################################
##############################################



#this sub will be called by sub re_predict_PREs_for_merged_regions
#start of sub re_predict
##############################################
##############################################
##############################################
sub re_predict  
    {	
        my$sequenceTitle_value=shift;
        my$prediction_sequence=shift;
        my$prediction_sequence_reverse_complement=shift;
        my$prediction_sequence_length=shift;
        my$PRE_id_value=shift;
        my$start_value=shift;
        my$end_value=shift;
        my$gap_to_print_value=shift;
        my$table_output_file_name=shift;
        my$detailed_output_file_name=shift;
        my$sequence_out_file_name=shift;
        
        my$dif_kind_motif=0;
        my$motif_name1='';
        my$motif_name2='';
        my$motif_gap='';
        my$cluster_start=0;
        my$cluster_stop=0;
        my$cluster_length=0;
        my$average_gap='';
        
	
	
     
	#for GAGA 
	my$gaga_found1=0;
	my$gaga_found2=0;
	my$gaga_found3=0;
	
	
	#for DSP1
	my$DSP1_found1=0;
	my$DSP1_found2=0;
	my$DSP1_found3=0;
	
	#for Zeste
	my$Zeste_found1=0;
	my$Zeste_found2=0;
	my$Zeste_found3=0;
        
	#for Su_HW
	my$Sp1_found1=0;
	my$Sp1_found2=0;
	my$Sp1_found3=0;
        
	#for Su_HW
	my$Pho_found1=0;
	my$Pho_found2=0;
	my$Pho_found3=0;
	
	

	
	#for EN1
	my$EN1_found1=0;
	my$EN1_found2=0;
	my$EN1_found3=0;
	
	
	
	#for ADF1
	my$ADF1_found1=0;
	my$ADF1_found2=0;
	my$ADF1_found3=0;
	
	#for ADF1
	my$Combgap_found1=0;
	my$Combgap_found2=0;
	my$Combgap_found3=0;
	
	
	 
	
	
	#for ADF1
	my$Grh_1_found1=0;
	my$Grh_1_found2=0;
	my$Grh_1_found3=0;
	
	
	#for ADF1
	my$Grh_2_found1=0;
	my$Grh_2_found2=0;
	my$Grh_2_found3=0;
	
	
	
	
        my$motif_name='';
        my$firstMatchingPosition1=0;
        my$lastMatchingPosition1=0;
        my$firstMatchingPosition2=0;
        my$lastMatchingPosition2=0;
        my$numberOfMatches=0;
      
        
       
	
        #for total number of motifs with out overlapping Su(Hw) sites
	my$total_number_of_motifs_after_removing_overlapping=0;
	
	#An array to hold results
	my @resultsArray=();
	
	#The pattern to match											   
	my @PRE_motifs = ("GAAAA","GAGAG","[C/T]GAG[C/T]G","[G/A][G/A]GG[T/C]G","C[A/T/G/C]GCCAT[A/T/G/C][A/T/G][A/T/G/C][A/T/G/C][A/T/G]","G[C/G][A/T/G/C][A/C]ACGCCCC","[G/T]C[G/A][G/A]C[G/A]GC[G/A][G/A]C[G/A]","GTGTGT","TGTTTTTT","[A/T]C[A/T/C]GGTT");
					
       
	
        
        foreach my$PRE_motifs(@PRE_motifs) 
        {
		
		#Removes the first element in the array and assign it to pattern variable
		my $pattern = $PRE_motifs;
                
		my $strandSearched;
		my $matchingSequence;
		my $firstMatchingPosition;
		my $lastMatchingPosition;
                
		#Perform the search for the direct strand first.
		
		$strandSearched = "+";
                
		while ($prediction_sequence =~ m/($pattern)/gi) 	
		{
			
    			$matchingSequence = $1;
    			$lastMatchingPosition = pos($prediction_sequence);
    			
			$firstMatchingPosition = $lastMatchingPosition - length($matchingSequence)+ 1;
			
			
                        
			
			
			if ($pattern eq "GAAAA")
    			{$DSP1_found1=$DSP1_found1+1;$motif_name="DSP1";}
			
			elsif ($pattern eq "GAGAG")
    			{$gaga_found1=$gaga_found1+1;$motif_name="GAF";}
			
			elsif ($pattern eq "[C/T]GAG[C/T]G")
    			{$Zeste_found1=$Zeste_found1+1;$motif_name="Zeste";}
			
			elsif ($pattern eq "[G/A][G/A]GG[T/C]G")
    			{$Sp1_found1=$Sp1_found1+1;$motif_name="Sp1";}
			
			elsif ($pattern eq "C[A/T/G/C]GCCAT[A/T/G/C][A/T/G][A/T/G/C][A/T/G/C][A/T/G]")
    			{$Pho_found1=$Pho_found1+1;$motif_name="Pho";}
			
			
                        
			elsif ($pattern eq "G[C/G][A/T/G/C][A/C]ACGCCCC")
    			{$EN1_found1=$EN1_found1+1;$motif_name="EN1";}
				
				
			elsif ($pattern eq "[G/T]C[G/A][G/A]C[G/A]GC[G/A][G/A]C[G/A]")
                    {$ADF1_found1=$ADF1_found1+1;$motif_name="ADF1";}		
                
				
			elsif ($pattern eq "GTGTGT")
                    {$Combgap_found1=$Combgap_found1+1;$motif_name="Combgap";}
					
			
			elsif ($pattern eq "TGTTTTTT")
				{$Grh_1_found1=$Grh_1_found1+1;$motif_name="Grh-1";}
					
			elsif ($pattern eq "[A/T]C[A/T/C]GGTT")
				{$Grh_2_found1=$Grh_2_found1+1;$motif_name="Grh-2";}
					
				
					
			$firstMatchingPosition1=$firstMatchingPosition+$start_value;
			$lastMatchingPosition1=$lastMatchingPosition+$start_value;
			
			
			
			
			push( @resultsArray, "$motif_name\t\t$firstMatchingPosition1\t$lastMatchingPosition1\t$strandSearched\t$matchingSequence\t\t");   
			
                        
                }#end of while loop for each + strand match
		
		
		
		
		$strandSearched = "-";
		
                
                
		while ($prediction_sequence_reverse_complement =~ m/($pattern)/gi) 
			
		{
   			
                        $matchingSequence = $1;
    			
			#These calculations make the positions correspond to the direct strand.
    			$firstMatchingPosition = $prediction_sequence_length - pos($prediction_sequence_reverse_complement) + 1;
    			$lastMatchingPosition = $firstMatchingPosition + length($matchingSequence)- 1;
    			
			
			if ($pattern eq "GAAAA")
    			{$DSP1_found2=$DSP1_found2+1;$motif_name="DSP1";}
			
			elsif ($pattern eq "GAGAG")
    			{$gaga_found2=$gaga_found2+1;$motif_name="GAF";}
			
			elsif ($pattern eq "[C/T]GAG[C/T]G")
    			{$Zeste_found2=$Zeste_found2+1;$motif_name="Zeste";}
			
			elsif ($pattern eq "[G/A][G/A]GG[T/C]G")
    			{$Sp1_found2=$Sp1_found2+1;$motif_name="Sp1";}
			
			elsif ($pattern eq "C[A/T/G/C]GCCAT[A/T/G/C][A/T/G][A/T/G/C][A/T/G/C][A/T/G]")
    			{$Pho_found2=$Pho_found2+1;$motif_name="Pho";}
			
                elsif ($pattern eq "G[C/G][A/T/G/C][A/C]ACGCCCC")
    			{$EN1_found2=$EN1_found2+1;$motif_name="EN1";}				
                
				
				elsif ($pattern eq "[G/T]C[G/A][G/A]C[G/A]GC[G/A][G/A]C[G/A]")
                {$ADF1_found2=$ADF1_found2+1;$motif_name="ADF1";}
				            
                elsif ($pattern eq "GTGTGT")
                {$Combgap_found2=$Combgap_found2+1;$motif_name="Combgap";}
				
				
				elsif ($pattern eq "TGTTTTTT")
				{$Grh_1_found2=$Grh_1_found2+1;$motif_name="Grh-1";}
					
			elsif ($pattern eq "[A/T]C[A/T/C]GGTT")
				{$Grh_2_found2=$Grh_2_found2+1;$motif_name="Grh-2";}
					
					
			
				
                        
			$firstMatchingPosition2=$firstMatchingPosition+$start_value;
			$lastMatchingPosition2=$lastMatchingPosition+$start_value;
			
			
			
			
				
			push( @resultsArray, "$motif_name\t\t$firstMatchingPosition2\t$lastMatchingPosition2\t$strandSearched\t$matchingSequence\t\t"); 
				
			
			
			
                        
                }#end of while loop for - strand each match
                
		
                
                
                
	}#end motif search loop 			      
	
        
        
	#Sort the results by start base using a map-sort-map.
	@resultsArray = map { $_->[1] }
      	sort { $a->[0] <=> $b->[0]}
      	map { [ getStartBase6($_), $_] }
      	@resultsArray;
	
	sub getStartBase6 	
	{
                my $result = shift();
                $result =~ m/^\S+\t\t(\d+)\t\d+\t[\+\-]\t\S+/;
                return $1;
	}
	
        
        
	# to find no of element in the array
        $numberOfMatches = scalar(@resultsArray);
	
        #to count overlapping Su(Hw) sites
	my$Pho_overlapping_site=0;	
        
	# to find the cluster start and end 
	my$cluster_start_line=$resultsArray[0];
	my$cluster_end_line=$resultsArray[-1];
	
	if ($cluster_start_line =~ m/^\S+\t\t(\d+)\t\d+\t[\+\-]\t\S+/)
	{$cluster_start=$1;}
	
	if ($cluster_end_line =~ m/^\S+\t\t\d+\t(\d+)\t[\+\-]\t\S+/)
	{$cluster_stop=$1;}
	
    $cluster_length=$cluster_stop-$cluster_start+1;			
        
        my$potential_Zeste_binding_sites=0;		
	my$potential_DSP1_binding_sites=0;
        
        # to add the gap detail to the final file 
	my@resultsArray1=();	
	
	#to push the first element in the array with 0 gap
	push( @resultsArray1, "$cluster_start_line\t0\n"); 
	
	#to find the gap for each motif
	for (my $i = 0 ; $i <= $numberOfMatches-2; $i++ ) 
        {
                
		my$current_first_motif_stop=0;
		my$current_second_motif_start=0;
		
		# to find the cluster start and end
		my$first_motif_start_line=$resultsArray[$i];
		my$second_motif_start_line=$resultsArray[$i+1];
		
		
		if ($first_motif_start_line =~ m/^(\S+)\t\t\d+\t(\d+)\t[\+\-]\t\S+/)
		{
                        $motif_name1=$1;
			$current_first_motif_stop=$2;
		}
		
		
		
		if ($second_motif_start_line =~ m/^(\S+)\t\t(\d+)\t\d+\t[\+\-]\t\S+/)
		{
                        $motif_name2=$1;
			$current_second_motif_start=$2;
			
                }
                
                
                $motif_gap=$current_second_motif_start-$current_first_motif_stop;
                
		                
                
                #to push the remaining elements in the array with gap
                push( @resultsArray1, "$second_motif_start_line\t$motif_gap\n"); 
                
        }#close of gap finding for loop	
	
        
        
	$dif_kind_motif=0;
	#To list no of motifs found  
        
	
        
	$DSP1_found3=$DSP1_found1+$DSP1_found2;
	if($DSP1_found3>=1)
	{$dif_kind_motif=$dif_kind_motif+1;}
	
	$gaga_found3=$gaga_found1+$gaga_found2;
	if($gaga_found3>=1)
	{$dif_kind_motif=$dif_kind_motif+1;}
        
	$Zeste_found3=$Zeste_found1+$Zeste_found2;
	if($Zeste_found3>=1)
	{$dif_kind_motif=$dif_kind_motif+1;}
	
	$Sp1_found3=$Sp1_found1+$Sp1_found2;
	if(($Sp1_found3>=1)or ($Pho_found3>=1))
	{$dif_kind_motif=$dif_kind_motif+1;}
     
	$Pho_found3=$Pho_found1+$Pho_found2;   
	if($Pho_found3>=1)
	{$dif_kind_motif=$dif_kind_motif+1;}
	
	
	$EN1_found3=$EN1_found1+$EN1_found2;
	if($EN1_found3>=1)
	{$dif_kind_motif=$dif_kind_motif+1;}
	
	$ADF1_found3=$ADF1_found1+$ADF1_found2;
	if($ADF1_found3>=1)
	{$dif_kind_motif=$dif_kind_motif+1;}
		
		
	$Combgap_found3=$Combgap_found1+$Combgap_found2;
	if($Combgap_found3>=1)
	{$dif_kind_motif=$dif_kind_motif+1;}
		
	$Grh_1_found3=$Grh_1_found1+$Grh_1_found2;
	$Grh_2_found3=$Grh_2_found1+$Grh_2_found2;

	
	
	
	
	if($numberOfMatches==0 or $cluster_length==0)
	{
                $average_gap="Not Applicaple";
                
	}
	
	else
	{
                $average_gap=($cluster_length/$numberOfMatches);
                $average_gap= sprintf("%.2f", $average_gap);
        	
                
		
                
		$total_number_of_motifs_after_removing_overlapping=$numberOfMatches;			
		
		      
        }#end of score calculation loop
        
        
         ###printing output file 
        open(OUTFILE, "+>>" . $detailed_output_file_name) or die ("Cannot open file for output: $!");
        
	print(OUTFILE "*********************************** MOTIF & PRE SUMMARY **************************\n\n");
        print(OUTFILE "PRE ID: $PRE_id_value\t\tLOCATION: $cluster_start - $cluster_stop\t\tSIZE: $cluster_length bp\n\n");	
	print(OUTFILE "\t\t\tTOTAL NO MOTIFS: $numberOfMatches\t\tAVERAGE GAP: $average_gap\t\n\n");
        print(OUTFILE "Pho: $Pho_found3\t\t\tGAF: $gaga_found3\t\tDSP1: $DSP1_found3\t\tZeste: $Zeste_found3\n\n");
        print(OUTFILE "Sp1: $Sp1_found3\t\t\tCombgap: $Combgap_found3\tEN1: $EN1_found3\t\tADF1: $ADF1_found3\n\n");
        print(OUTFILE "Grh-1: $Grh_1_found3\t\tGrh-2: $Grh_2_found3\n\n");
		
		
        
        #print(OUTFILE "SEQUENCE SEGMENT USED: $start_value- $end_value\n\n");

        print(OUTFILE "**********************************************************************************\n");
	print(OUTFILE "Motif\t\tStart\tStop\tStrand\tSequence found\t\tGap\n\n");
        
        foreach (@resultsArray1) 
        {print (OUTFILE $_);}
	
	print(OUTFILE "----------------------------------------------------------------------------------\n\n\n");
        close(OUTFILE) or die ("Cannot close file for output: $!");
	
        #end of printing for detailed output 

          
        ################################################## to print the output to the file1 in consice format
	open(OUTFILE, "+>>" . $table_output_file_name) or die ("Cannot open file for output: $!");
	print(OUTFILE "$PRE_id_value\t$cluster_start - $cluster_stop\t\t$cluster_length\t\t$numberOfMatches\t$gap_to_print_value\n");
	close(OUTFILE) or die ("Cannot close file for output: $!");
        #end of printing for a given seq

	################################################## this output file is used for graphics creation
    
        #to print output file which is used for graphic file creation
        open(OUTFILE, "+>>" . $table_to_graphics_output_file_name) or die ("Cannot open file for output: $!");
       print(OUTFILE "PRE\t$PRE_id_value\t$cluster_start\t$cluster_stop\n");
        close(OUTFILE) or die ("Cannot close file for output: $!");
        #end of printing for output file which is used for graphic file creation


        ##################################################to print the sequence in separate file.
	open(OUTFILE, "+>>" . $sequence_out_file_name) or die ("Cannot open file for output: $!");
	print(OUTFILE ">PRE ID: $PRE_id_value Location $cluster_start - $cluster_stop ($cluster_length bp) $sequenceTitle_value\n");
        print(OUTFILE "$prediction_sequence\n\n");
	close(OUTFILE) or die ("Cannot close file for output: $!");
        #end of printing for a given seq
   
     
   
 
}	      
#end of sub re_predict
##############################################
##############################################
##############################################	


























#subroutine to get data from a file given its filename
##############################################
##############################################
##############################################
sub get_file_data
    {
        
        my($filename) = @_;
       
        
        # Initialize variables
        my @filedata = (  );
        
        unless( open(GET_FILE_DATA, $filename) )
        {
        print STDERR "Cannot open file \"$filename\"\n\n";
        exit;
        }
        
        @filedata = <GET_FILE_DATA>;
        
        close GET_FILE_DATA;
        
        return @filedata;
    }

##############################################
##############################################
##############################################








#subroutine to get data from a file given its filename
##############################################
##############################################
##############################################
sub get_file_data1
    {
        
        my($filename1) = @_;
       
        
        # Initialize variables
        my @filedata1 = (  );
        
        unless( open(GET_FILE_DATA, $filename1) )
        {
        print STDERR "Cannot open file \"$filename1\"\n\n";
        exit;
        }
        
        @filedata1 = <GET_FILE_DATA>;
        
        close GET_FILE_DATA;
        
        return @filedata1;
    }

##############################################
##############################################
##############################################

